pub enum TreeOptimizationMode {
Reinsert,
PartialRebuild,
FullRebuild,
Adaptive {
reinsert_threshold: f32,
partial_rebuild_threshold: f32,
},
}Expand description
The optimization mode for a ColliderTree.
Variants
Reinsert
The tree is optimized by reinserting proxies whose AABB in the tree has changed.
This is the fastest method when only a small portion of proxies have moved, but is less effective for large numbers of moved proxies.
PartialRebuild
The tree is optimized by performing a partial rebuild that only rebuilds parts of the tree affected by proxies that have moved.
This method is more effective than reinsertion when a moderate number of proxies have moved. However, if a large portion of proxies have moved, a full rebuild can be more effective and have less overhead.
FullRebuild
The tree is optimized by performing a full rebuild.
This method can produce the highest quality tree, and can have less overhead than other methods when a large portion of proxies have moved. This makes it suitable for highly dynamic scenes.
Adaptive
The tree is optimized adaptively based on how many proxies have moved.
- If the ratio of moved proxies to total proxies is below
reinsert_threshold,Reinsertis used. - If the ratio is between
reinsert_thresholdandpartial_rebuild_threshold,PartialRebuildis used. - Otherwise,
FullRebuildis used.
This is the default mode.
Implementations
Sourceimpl TreeOptimizationMode
impl TreeOptimizationMode
Sourcepub fn resolve(&self, moved_ratio: f32) -> TreeOptimizationMode
pub fn resolve(&self, moved_ratio: f32) -> TreeOptimizationMode
Resolves the optimization mode based on the ratio of moved proxies.
moved_ratio is the ratio of moved proxies to total proxies in the tree.