ColliderTree

Struct ColliderTree 

pub struct ColliderTree {
    pub bvh: Bvh2,
    pub proxies: StableVec<ColliderTreeProxy>,
    pub moved_proxies: Vec<ProxyId>,
    pub workspace: ColliderTreeWorkspace,
}
A Bounding Volume Hierarchy (BVH) for accelerating queries on a set of colliders.

See the collider_tree module for more information.

Fields

bvh: Bvh2

The underlying BVH structure.

proxies: StableVec<ColliderTreeProxy>

The proxies stored in the tree.

moved_proxies: Vec<ProxyId>

A list of moved proxies since the last update.

This is used during tree optimization to determine which proxies need to be reinserted or rebuilt.

workspace: ColliderTreeWorkspace

A workspace for reusing allocations across tree operations.

Implementations

Source

impl ColliderTree

pub fn ray_traverse_closest<F: FnMut(ProxyId) -> Scalar>( &self, ray: Ray3d, max_distance: Scalar, intersection_fn: F, ) -> Option<(ProxyId, Scalar)>

Traverses the tree for the closest intersection with the given ray.

Arguments
  • ray: The ray to be tested for intersection.
  • max_distance: The maximum distance along the ray to consider for intersections.
  • intersection_fn: A function that takes a proxy ID, and returns the distance to the intersection with that proxy. This function is called for each potential intersection found during traversal.
pub fn ray_traverse_all<F: FnMut(ProxyId) -> bool>( &self, ray: Ray3d, max_distance: Scalar, intersection_fn: F, )

Traverses the tree for all intersections with the given ray.

Terminates when all intersections within max_distance have been visited or when intersection_fn returns false for an intersection.

Arguments
  • ray: The ray to be tested for intersection.
  • max_distance: The maximum distance along the ray to consider for intersections.
  • intersection_fn: A function that takes a proxy ID, and is called for each potential intersection found during traversal. Return false to halt traversal early.
pub fn sweep_traverse_closest<F: FnMut(ProxyId) -> Scalar>( &self, aabb: Aabb, direction: Dir, max_distance: Scalar, target_distance: Scalar, intersection_fn: F, ) -> Option<(ProxyId, Scalar)>

Traverse the BVH by sweeping an AABB along a velocity vector, returning the closest hit.

Arguments
  • aabb: The axis-aligned bounding box to be swept.
  • direction: The direction along which to sweep the AABB.
  • target_distance: The separation distance at which a hit is still considered valid.
  • max_distance: The maximum distance along the sweep to consider for intersections.
  • intersection_fn: A function that takes a proxy ID, and returns the distance to the intersection with that proxy. This function is called for each potential intersection found during traversal.
pub fn sweep_traverse_all<F: FnMut(ProxyId) -> bool>( &self, aabb: Aabb, direction: Dir, target_distance: Scalar, max_distance: Scalar, intersection_fn: F, )

Traverse the BVH by sweeping an AABB along a velocity vector, calling intersection_fn for each hit.

Arguments
  • aabb: The axis-aligned bounding box to be swept.
  • direction: The direction along which to sweep the AABB.
  • target_distance: The separation distance at which a hit is still considered valid.
  • max_distance: The maximum distance along the sweep to consider for intersections.
  • intersection_fn: A function that takes a proxy ID, and is called for each potential intersection found during traversal. Return false to halt traversal early.
pub fn squared_distance_traverse_closest<F: FnMut(ProxyId) -> Scalar>( &self, point: Vector, max_distance_squared: Scalar, eval: F, ) -> Option<(ProxyId, Scalar)>

Traverse the BVH with a point, returning the closest proxy and its squared distance within max_distance_squared.

Arguments
  • point: The point to be tested for proximity.
  • max_distance_squared: The maximum distance from the point to consider for projections.
  • eval: A function that takes a proxy ID and returns the squared distance from the point to that proxy. This function is called for each potential projection found during traversal.
pub fn point_traverse<F: FnMut(ProxyId) -> bool>(&self, point: Vector, eval: F)

Traverse the BVH with a point, calling eval for each intersection.

Arguments
  • point: The point to be tested for intersection.
  • eval: A function that takes a proxy ID and is called for each potential intersection found during traversal. Return false to halt traversal early.
pub fn aabb_traverse<F: FnMut(ProxyId) -> bool>(&self, aabb: Aabb, eval: F)

Traverse the BVH with an AABB, calling eval for each intersection.

Arguments
  • aabb: The axis-aligned bounding box to be tested for intersection.
  • eval: A function that takes a proxy ID and is called for each potential intersection found during traversal. Return false to halt traversal early.
Source

impl ColliderTree

pub fn add_proxy(&mut self, aabb: Aabb, proxy: ColliderTreeProxy) -> ProxyId

Adds a proxy to the tree, returning its index.

pub fn remove_proxy(&mut self, proxy_id: ProxyId) -> Option<ColliderTreeProxy>

Removes a proxy from the tree.

Returns true if the proxy was successfully removed, or false if the proxy ID was invalid.

pub fn get_proxy(&self, proxy_id: ProxyId) -> Option<&ColliderTreeProxy>

Gets a proxy from the tree by its ID.

Returns None if the proxy ID is invalid.

pub fn get_proxy_mut( &mut self, proxy_id: ProxyId, ) -> Option<&mut ColliderTreeProxy>

Gets a mutable reference to a proxy from the tree by its ID.

Returns None if the proxy ID is invalid.

pub unsafe fn get_proxy_unchecked( &self, proxy_id: ProxyId, ) -> &ColliderTreeProxy

Gets a proxy from the tree by its ID without bounds checking.

Safety

The caller must ensure that the proxy_id is valid.

pub unsafe fn get_proxy_unchecked_mut( &mut self, proxy_id: ProxyId, ) -> &mut ColliderTreeProxy

Gets a mutable reference to a proxy from the tree by its ID without bounds checking.

Safety

The caller must ensure that the proxy_id is valid.

pub fn get_proxy_aabb(&self, proxy_id: ProxyId) -> Option<Aabb>

Gets the AABB of a proxy in the tree.

Returns None if the proxy ID is invalid.

pub unsafe fn get_proxy_aabb_unchecked(&self, proxy_id: ProxyId) -> Aabb

Gets the AABB of a proxy in the tree without bounds checking.

Safety

The caller must ensure that the proxy_id is valid.

pub fn set_proxy_aabb(&mut self, proxy_id: ProxyId, aabb: Aabb)

Updates the AABB of a proxy in the tree.

If the BVH should be refitted at the same time, consider using resize_proxy_aabb instead.

If resizing a large number of proxies, consider calling this method for each proxy and then calling refit_all once at the end.

pub fn resize_proxy_aabb(&mut self, proxy_id: ProxyId, aabb: Aabb)

Resizes the AABB of a proxy in the tree.

This is equivalent to calling set_proxy_aabb and then refitting the BVH working up from the resized node.

For resizing a large number of proxies, consider calling set_proxy_aabb for each proxy and then calling refit_all once at the end.

pub fn reinsert_proxy(&mut self, proxy_id: ProxyId, aabb: Aabb)

Updates the AABB of a proxy and reinserts it at an optimal place in the tree.

pub fn refit_all(&mut self)

Refits the entire tree from the leaves up.

pub fn rebuild_full(&mut self)

Fully rebuilds the tree from the given list of AABBs.

pub fn rebuild_partial(&mut self, leaves: &[u32])

Rebuilds parts of the tree corresponding to the given list of leaf node indices.

pub fn optimize(&mut self, batch_size_ratio: f32)

Restructures the tree using parallel reinsertion, optimizing node locations based on SAH cost.

This can be used to improve query performance after the tree quality has degraded, for example after many proxy insertions and removals.

pub fn optimize_candidates(&mut self, candidates: &[u32], iterations: u32)

Restructures the tree using parallel reinsertion, optimizing node locations based on SAH cost.

Only the specified candidate proxies are considered for reinsertion.