pub struct ColliderTreeOptimization {
pub optimization_mode: TreeOptimizationMode,
pub optimize_in_place: bool,
pub use_async_tasks: bool,
}Expand description
Settings for optimizing each ColliderTree.
Fields
optimization_mode: TreeOptimizationModeThe optimization mode for the collider tree.
Default: TreeOptimizationMode::Adaptive
optimize_in_place: boolIf true, tree optimization will be performed in-place with minimal allocations.
This has the downside that the tree will be unavailable for spatial queries
during the simulation step while the optimization is ongoing (ex: in collision hooks).
Otherwise, parts of the the tree will be cloned for the optimization, allowing spatial queries to use the old tree during the simulation step, but incurring additional memory allocation overhead.
For optimal performance, set this to true if your application
does not perform spatial queries during the simulation step.
Default: false
use_async_tasks: boolIf true, tree optimization will be performed in parallel
with the narrow phase and solver using async tasks.
Default: true (on supported platforms)