pub struct ColliderTreeProxy {
pub collider: Entity,
pub body: Option<Entity>,
pub layers: CollisionLayers,
pub flags: ColliderTreeProxyFlags,
}Expand description
A proxy representing a collider in the ColliderTree.
Fields
collider: EntityThe collider entity this proxy represents.
body: Option<Entity>The body this collider is attached to.
layers: CollisionLayersThe collision layers of the collider.
flags: ColliderTreeProxyFlagsFlags for the proxy.
Implementations
Sourceimpl ColliderTreeProxy
impl ColliderTreeProxy
Sourcepub fn has_custom_filter(&self) -> bool
pub fn has_custom_filter(&self) -> bool
Returns true if the custom filtering hook is active.
Sourcepub fn has_contact_modification(&self) -> bool
pub fn has_contact_modification(&self) -> bool
Returns true if the contact modification hook is active.