pub struct ColliderTrees {
pub dynamic_tree: ColliderTree,
pub kinematic_tree: ColliderTree,
pub static_tree: ColliderTree,
pub standalone_tree: ColliderTree,
}Expand description
Trees for accelerating queries on a set of colliders.
See the collider_tree module for more information.
Fields
dynamic_tree: ColliderTreeA tree for the colliders of dynamic bodies.
kinematic_tree: ColliderTreeA tree for the colliders of kinematic bodies.
static_tree: ColliderTreeA tree for the colliders of static bodies.
standalone_tree: ColliderTreeA tree for standalone colliders with no associated rigid body.
Implementations
Sourceimpl ColliderTrees
impl ColliderTrees
Sourcepub const fn tree_for_type(&self, tree_type: ColliderTreeType) -> &ColliderTree
pub const fn tree_for_type(&self, tree_type: ColliderTreeType) -> &ColliderTree
Returns the tree for the given ColliderTreeType.
Sourcepub const fn tree_for_type_mut(
&mut self,
tree_type: ColliderTreeType,
) -> &mut ColliderTree
pub const fn tree_for_type_mut( &mut self, tree_type: ColliderTreeType, ) -> &mut ColliderTree
Returns a mutable reference to the tree for the given ColliderTreeType.
Sourcepub fn iter_trees(&self) -> impl Iterator<Item = &ColliderTree>
pub fn iter_trees(&self) -> impl Iterator<Item = &ColliderTree>
Returns an iterator over all collider trees.
Sourcepub fn iter_trees_mut(&mut self) -> impl Iterator<Item = &mut ColliderTree>
pub fn iter_trees_mut(&mut self) -> impl Iterator<Item = &mut ColliderTree>
Returns a mutable iterator over all collider trees.
Sourcepub fn get_proxy(&self, key: ColliderTreeProxyKey) -> Option<&ColliderTreeProxy>
pub fn get_proxy(&self, key: ColliderTreeProxyKey) -> Option<&ColliderTreeProxy>
Returns the proxy with the given ColliderTreeProxyKey, if it exists.
Sourcepub fn get_proxy_mut(
&mut self,
key: ColliderTreeProxyKey,
) -> Option<&mut ColliderTreeProxy>
pub fn get_proxy_mut( &mut self, key: ColliderTreeProxyKey, ) -> Option<&mut ColliderTreeProxy>
Returns a mutable reference to the proxy with the given ColliderTreeProxyKey, if it exists.