pub struct MovedProxies { /* private fields */ }Expand description
A resource for tracking moved proxies.
Moved proxies are those whose ColliderAabb has moved outside of their
previous EnlargedAabb, or whose collider has been added to a ColliderTree.
Implementations
Sourceimpl MovedProxies
impl MovedProxies
Sourcepub fn proxies(&self) -> &[ColliderTreeProxyKey]
pub fn proxies(&self) -> &[ColliderTreeProxyKey]
Returns the keys of the moved proxies.
The order of the keys is the order in which they were inserted.
Sourcepub fn contains(&self, proxy_key: ColliderTreeProxyKey) -> bool
pub fn contains(&self, proxy_key: ColliderTreeProxyKey) -> bool
Returns true if the proxy with the given key has moved.
Sourcepub fn insert(&mut self, proxy_key: ColliderTreeProxyKey) -> bool
pub fn insert(&mut self, proxy_key: ColliderTreeProxyKey) -> bool
Inserts a moved proxy key.
Returns true if the proxy key was not already present.
Sourcepub fn remove(&mut self, proxy_key: &ColliderTreeProxyKey)
pub fn remove(&mut self, proxy_key: &ColliderTreeProxyKey)
Removes a moved proxy key. This uses a linear search, and may change the order of the remaining keys.
If the proxy key is not present, nothing happens.